# -*- coding: utf-8 -*-

from martian.util import isclass
from dolmen.content import schema
from dolmen.forms.crud.interfaces import IFieldsCustomization
from zeam.form.base import Fields
from zope.component import getGlobalSiteManager, queryMultiAdapter
from zope.interface import implementedBy, providedBy


def queryClassMultiAdapter(adapts, interface, factory_cls=None, name=u''):
    """This function searches the component registry for any adapter
    registered for an instance of the given class. It lookups and returns
    it correctly factored.
    """
    sm = getGlobalSiteManager()
    klass = adapts[0]

    if factory_cls is None:
        factory_cls = klass

    required = implementedBy(klass)
    lookfor = (required,) + tuple(providedBy(a) for a in adapts[1:])
    factory = sm.adapters.lookup(lookfor, interface, name)
    if factory is not None:
        return factory(factory_cls, *adapts[1:])
    return None


def getSchemaFields(form, component, *ignore):
    if isclass(component):
        lookup = queryClassMultiAdapter
    else:
        lookup = queryMultiAdapter

    ifaces = schema.bind(default=None).get(component)
    if ifaces:
        fields = Fields(*ifaces).omit(*ignore)
        modifier = lookup((component, form, form.request),
                          IFieldsCustomization)
        if modifier is not None:
            return modifier(fields)
        return fields
    return Fields()



# -*- coding: utf-8 -*-

import grokcore.component as grok
from dolmen.content import schema, IContent
from dolmen.forms.base import Fields, IFieldUpdate
from zope.component import getAdapters
from zope.lifecycleevent import IObjectModifiedEvent, IObjectCreatedEvent


@grok.subscribe(IContent, IObjectCreatedEvent)
def notify_fields_creation(ob, event):
    """This handler propagates the ObjectCreatedEvent to a more atomic
    level, by calling an adapter on each field of the schema. This permits
    to actually interact at the field level, after it gets a value for the
    first time.
    """
    schemas = schema.bind().get(ob)
    fields = Fields(*schemas)

    for field_repr in fields:
        field = field_repr._field
        if field.get(ob) != field.missing_value:
            handlers = getAdapters((ob, field), IFieldUpdate)
            for handler in handlers:
                # Iteration through the generator
                pass


@grok.subscribe(IContent, IObjectModifiedEvent)
def notify_fields_update(ob, event):
    """This handler propagates the ObjectModifiedEvent to a more atomic
    level, by calling an adapter on each modified field. This permits to
    actually interact at the field level, after it gets modified.
    """
    for desc in event.descriptions:
        for name in desc.attributes:
            field = desc.interface[name]
            handlers = getAdapters((ob, field), IFieldUpdate)
            for handler in handlers:
                # Iteration through the generator
                pass



# -*- coding: utf-8 -*-

import zope.i18n
import dolmen.content as content
import grokcore.component as grok

from dolmen.forms.base import ApplicationForm, DISPLAY
from dolmen.forms.crud import actions as formactions, i18n as _
from dolmen.forms.crud.interfaces import IFactoryAdding
from dolmen.forms.crud.utils import getSchemaFields

from zeam.form.base import Actions
from zope.cachedescriptors.property import CachedProperty
from zope.dublincore.interfaces import IDCDescriptiveProperties
from zope.i18nmessageid import Message


class Add(ApplicationForm):
    """The add form itself is not protected. The security is checked on
    'update'. It checks if the 'require' directive of the factored item
    is respected on the context.
    """
    grok.baseclass()
    grok.title(_(u"Add"))
    grok.name('dolmen.add')
    grok.context(IFactoryAdding)

    @property
    def label(self):
        name = self.context.factory.name
        if isinstance(name, Message):
            name = zope.i18n.translate(name, context=self.request)
        return zope.i18n.translate(
            _(u"add_action", default="Add: $name",
              mapping={'name': name}), context=self.request)

    @CachedProperty
    def fields(self):
        return getSchemaFields(
            self, self.context.factory.factory, '__parent__')

    @CachedProperty
    def actions(self):
        add = formactions.AddAction(_("Add"), self.context.factory)
        return Actions(add, formactions.CancelAction(_("Cancel")))


class Edit(ApplicationForm):
    grok.baseclass()
    grok.name('edit')
    grok.title(_(u"Edit"))
    grok.context(content.IContent)

    ignoreContent = False
    ignoreRequest = False
    actions = Actions(formactions.UpdateAction(_("Update")),
                      formactions.CancelAction(_("Cancel")))

    @property
    def label(self):
        label = _(u"edit_action", default=u"Edit: $name",
                  mapping={"name": self.context.__name__})
        return zope.i18n.translate(label, context=self.request)

    @CachedProperty
    def fields(self):
        return getSchemaFields(
            self, self.getContentData().getContent(),
            '__parent__')


class Display(ApplicationForm):
    grok.baseclass()
    grok.title(_(u"View"))
    grok.context(content.IContent)

    mode = DISPLAY
    ignoreRequest = True
    ignoreContent = False

    @property
    def label(self):
        dc = IDCDescriptiveProperties(self.context, None)
        if dc is not None and dc.title:
            return dc.title
        return getattr(self.context, '__name__', u'')

    @CachedProperty
    def fields(self):
        return getSchemaFields(
            self, self.getContentData().getContent(),
            '__parent__', 'title')


class Delete(ApplicationForm):
    """A confirmation for to delete an object.
    """
    grok.baseclass()
    grok.title(_(u"Delete"))
    grok.context(content.IContent)

    label = _(u"Delete")
    description = _(u"Are you really sure ?")
    actions = Actions(formactions.DeleteAction(_("Delete")),
                      formactions.CancelAction(_("Cancel")))



# -*- coding: utf-8 -*-

import zope.schema
import megrok.layout
import dolmen.content
import grokcore.component as grok
from zope.interface import Interface
from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import contains
from zope.dublincore.property import DCProperty


class IDesertWorm(dolmen.content.IContent):
    """Defines a gigantic worm living in the desert.
    """
    length = zope.schema.Float(
        title=u"Size of worm from head to tails in meters",
        default=300.0,
        required=True)


class IDesertWarrior(dolmen.content.IContent, IAttributeAnnotatable):
    """Defines a warrior living in the desert. Can be annotated.
    """
    title = zope.schema.TextLine(
        title=u"Name of the warrior",
        default=u"",
        required=True)

    water = zope.schema.Int(
        title=u"Number water gallons owned",
        default=1,
        required=True)


class IDesertCave(dolmen.content.IContent):
    """Defines a cave carved in the mountains by the Coriolis storms.
    """
    contains(IDesertWarrior)


class Sietch(dolmen.content.Container):
    """A grotto located on Arrakis.
    """
    dolmen.content.name('sietch')
    dolmen.content.schema(IDesertCave)
    dolmen.content.nofactory()


class TrainingCamp(dolmen.content.Factory):
    """A camp that produces Fremen warriors.
    """
    dolmen.content.name(u"fremen")
    title = u"Fremen training camp"


class Fremen(dolmen.content.Content):
    """Inhabitants on the deep deserts. They live in sietches.
    """
    dolmen.content.name(u'Fremen Warrior')
    dolmen.content.schema(IDesertWarrior)
    dolmen.content.factory(TrainingCamp)
    dolmen.content.require('zope.ManageContent')
    title = DCProperty('title')


class ShaiHulud(dolmen.content.Content):
    """Gigantic desert worm of Arrakis.
    """
    dolmen.content.name(u'Shai Hulud')
    dolmen.content.schema(IDesertWorm)
    dolmen.content.require('zope.ManageContent')
    dolmen.content.nofactory()


class Harkonnen(dolmen.content.Content):
    """A native of Giedi Prime.
    """
    dolmen.content.name('Harkonnen')
    dolmen.content.nofactory()


class MyLayout(megrok.layout.Layout):
    grok.context(Interface)

    def render(self):
        return self.view.content()



from os.path import join
from setuptools import setup, find_packages

name = 'dolmen.forms.crud'
version = '1.0b3'
readme = open(join('src', 'dolmen', 'forms', 'crud', 'README.txt')).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'dolmen.content >= 0.7',
    'dolmen.forms.base >= 1.0b1',
    'grokcore.component',
    'setuptools',
    'zeam.form.base',
    'zeam.form.layout',
    'zeam.form.ztk',
    'zope.cachedescriptors',
    'zope.component >= 3.9.1',
    'zope.container',
    'zope.dublincore',
    'zope.event',
    'zope.i18n',
    'zope.i18nmessageid',
    'zope.interface',
    'zope.lifecycleevent',
    'zope.location',
    'zope.publisher',
    'zope.schema',
    'zope.security',
    'zope.traversing',
    ]

tests_require = [
    'grokcore.message',
    'megrok.layout',
    'zope.annotation',
    'zope.i18n',
    'zope.principalregistry',
    'zope.securitypolicy',
    'zope.session',
    'zope.site',
    ]

setup(name = name,
      version = version,
      description = "CRUD forms and actions for `dolmen.content`",
      long_description = readme + '\n\n' + history,
      keywords = 'Dolmen Grok Content Forms',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = 'http://www.dolmen-project.org',
      license = 'GPL',
      packages = find_packages('src', exclude=['ez_setup']),
      package_dir = {'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.forms'],
      include_package_data = True,
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite = "dolmen.forms.crud",
      classifiers = [
          'Development Status :: 4 - Beta',
          'Environment :: Web Environment',
          'Framework :: Zope3',
          'Intended Audience :: Other Audience',
          'License :: OSI Approved :: GNU General Public License (GPL)',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          ],
      )



# -*- coding: utf-8 -*-

import doctest
import unittest
import zope.component

from dolmen.forms.crud import tests
from zope.component import eventtesting
from zope.component.interfaces import IComponentLookup
from zope.component.testlayer import ZCMLFileLayer
from zope.interface import Interface
from zope.site.folder import rootFolder
from zope.site.site import LocalSiteManager, SiteManagerAdapter
from zope.traversing.testing import setUp as traversingSetUp


class DolmenFormsCrudLayer(ZCMLFileLayer):
    """The dolmen.forms.crud main test layer.
    """

    def setUp(self):
        ZCMLFileLayer.setUp(self)
        eventtesting.setUp()
        traversingSetUp()
        zope.component.hooks.setHooks()

        # Set up site manager adapter
        zope.component.provideAdapter(
            SiteManagerAdapter, (Interface,), IComponentLookup)

        # Set up site
        site = rootFolder()
        site.setSiteManager(LocalSiteManager(site))
        zope.component.hooks.setSite(site)

    def tearDown(self):
        ZCMLFileLayer.tearDown(self)
        zope.component.hooks.resetHooks()
        zope.component.hooks.setSite()


def test_suite():
    suite = unittest.TestSuite()
    readme = doctest.DocFileSuite(
        '../README.txt', globs={'__name__': 'dolmen.forms.crud.tests'},
        optionflags=(doctest.ELLIPSIS + doctest.NORMALIZE_WHITESPACE))
    readme.layer = DolmenFormsCrudLayer(tests)
    suite.addTest(readme)
    return suite
